$ifndef _HBUS1_INC_
$define _HBUS1_INC_
'
' Routines to communicate via I2C using the MSSP1 peripheral
' Written by Les Johnson for the Proton BASIC Compiler
'
' Setup the default pins used for the MSSP1 peripheral
'
$ifndef Hbus1_SCL_Pin
    $define Hbus1_SCL_Pin PORTC.3
$endif
$ifndef Hbus1_SDA_Pin
    $define Hbus1_SDA_Pin PORTC.4
$endif
'
' Create some variables for the Hbus routines
'
    Dim HbusX_wTimeout As Word System           ' Used for timeout counting
    Dim HbusX_bTemp As Byte System              ' A temporary byte variable
    Dim HbusX_bFlags As Byte System             ' Used for Error flags

$define Hbus1_Error HbusX_bFlags.2              ' Set to 1 if an error occurs
$define Hbus1_tReadWrite HbusX_bFlags.1         ' Set to 1 for read and 0 for write

$define Hbus1_Write 0                           ' Set slave device to write mode
$define Hbus1_Read 1                            ' Set slave device to read mode

$if (_mssp = 1) Or (_mssp = 2)                  ' This set of routines if the device has an MSSP peripheral
    GoTo _Hbus1_Main_                           ' Jump over the routines to the main start
'-----------------------------------------------------------------------------------------------
$define Hbus1_Delay() DelayUS 1

'-----------------------------------------------------------------------------------------------
' Configure the MSSP for master I2C mode
' Input     : None
' Output    : None
' Notes     : None
'
$define Hbus1_Setup() _Hbus1_Setup

_Hbus1_Setup Macro-
    GoSub __Hbus1_Setup
Endm

#ifMacro _Hbus1_Setup
__Hbus1_Setup:
    Input Hbus1_SCL_Pin                         ' Make SCL an input
    Input Hbus1_SDA_Pin                         ' Make SDA an input
    SSPSTAT = 128                               ' Disable the slew rate for normal 100KHz operation
                                                ' Set the input levels to conform to I2C specs
    SSPADD = ((Xtal * 1000) / (100 * 4)) - 1    ' Calculate the Baud rate value for SSPADD
    SSPCON1 = 40                                ' I2C master mode selected and Enable the SSP
    Return
#endIfMacro

'-----------------------------------------------------------------------------------------------
' Wait for the I2C bus to become idle
' Input     : None
' Output    : Bit Hbus1_Error holds 1 if failure, else 0
' Notes     : None
'
$define Hbus1_Wait_For_Idle() _Hbus1_Wait_For_Idle

_Hbus1_Wait_For_Idle Macro-
    GoSub __Hbus1_Wait_For_Idle
Endm

#ifMacro- _Hbus1_Wait_For_Idle
__Hbus1_Wait_For_Idle:
    HbusX_wTimeout = $FFFF                  ' Use HbusX_wTimeout as a counting loop
    Repeat
        Hbus1_Delay()                       ' A small delay
        Clear Hbus1_Error                   ' Default to Acknowledge received
        If Byte SSPCON2 & %00011111 = 0 Then Return ' Zero so I2C bus is ready, so exit with Hbus1_Error clear
        Dec HbusX_wTimeout
    Until HbusX_wTimeout = 0
    Set Hbus1_Error                         ' Set Hbus1_Error to signal no Acknowledge received
    Return
#endIfMacro

'-----------------------------------------------------------------------------------------------
' Generate an I2C Start
' Input     : None
' Output    : None
' Notes     : None
'
$define Hbus1_Start()   '
    Set SSPCON2bits_SEN '
    While SSPCON2bits_SEN = 1: Wend

'-----------------------------------------------------------------------------------------------
' Generate an I2C Stop
' Input     : None
' Output    : None
' Notes     : None
'
$define Hbus1_Stop()     '
    Set SSPCON2bits_PEN  '
    While SSPCON2bits_PEN = 1: Wend

'-----------------------------------------------------------------------------------------------
' Generate an I2C ReStart
' Input     : None
' Output    : None
' Notes     : None
'
$define Hbus1_ReStart()  '
    Set SSPCON2bits_RSEN '
    While SSPCON2bits_RSEN = 1: Wend

'-----------------------------------------------------------------------------------------------
' Generate an I2C Nack
' Input     : None
' Output    : None
' Notes     : None
'
$define Hbus1_Nack() _Hbus1_Nack

_Hbus1_Nack Macro-
    GoSub __Hbus1_Nack
Endm

#ifMacro- _Hbus1_Nack
__Hbus1_Nack:
    Set SSPCON2bits_ACKDT               ' Set acknowledge bit for Not Ack
    Set SSPCON2bits_ACKEN               ' Initiate bus Not acknowledge sequence
    While SSPCON2bits_ACKEN = 1: Wend   ' Check if we are in idle mode
    Return
#endIfMacro

'-----------------------------------------------------------------------------------------------
' Generate an I2C Ack
' Input     : None
' Output    : None
' Notes     : None
'
$define Hbus1_Ack() _Hbus1_Ack

_Hbus1_Ack Macro-
    GoSub __Hbus1_Ack
Endm

#ifMacro- _Hbus1_Ack
__Hbus1_Ack:
    Clear SSPCON2bits_ACKDT                ' Clear acknowledge bit for Ack
    Set SSPCON2bits_ACKEN  				   ' Send an Ack condition
    While SSPCON2bits_ACKEN = 1: Wend
    Return
#endIfMacro

'-----------------------------------------------------------------------------------------------
' Send a byte via I2C
' Input     : pByteout (SSPBUF) holds the byte to write
' Output    : Bit Hbus1_Error holds 1 If failure, Else 0
' Notes     : None
'
$define Hbus1_Write_Byte(pByteout)  '
    SSPBUF = pByteout               '
    While SSPSTATbits_R_W = 1: Wend '
    Hbus1_Wait_For_Idle()

'-----------------------------------------------------------------------------------------------
' Send a slave byte via I2C
' Input     : pSlaveAddress holds the I2C slave address
'           : pReadWrite holds 1 or 0 for read or write (1 = read, 0 = write)
' Output    : Bit Hbus1_Error holds 1 If failure, Else 0
' Notes     : Shifts the address 1 bit left, then sets or clears the read/write bit
'
$define Hbus1_Set_SlaveAddress(pSlaveAddress, pReadWrite) '
    HbusX_bTemp = pSlaveAddress                       '
    Hbus1_tReadWrite = pReadWrite                     '
    _Hbus1_Set_SlaveAddress

_Hbus1_Set_SlaveAddress Macro-
    GoSub __Hbus1_Set_SlaveAddress
Endm

#ifMacro- _Hbus1_Set_SlaveAddress
__Hbus1_Set_SlaveAddress:
    HbusX_bTemp = HbusX_bTemp << 1
    If Hbus1_tReadWrite = Hbus1_Read Then
        Set HbusX_bTemp.0
    EndIf
    SSPBUF = HbusX_bTemp
    While SSPSTATbits_R_W = 1: Wend            ' Check if data transmit is complete
    Hbus1_Wait_For_Idle()
    Return
#endIfMacro

'-----------------------------------------------------------------------------------------------
' Receive a byte via I2C
' Input     : None
' Output    : pResult (SSPBUF) holds the byte received
' Notes     : None
'
$define Hbus1_Read_Byte(pResult) '
    Set SSPCON2bits_RCEN         '
    Hbus1_Wait_For_Idle()        '
    pResult = SSPBUF

'-----------------------------------------------------------------------------------------------
' Setup the MSSP1 peripheral
'
_Hbus1_Main_:
    Hbus1_Setup()

$else
    $error "Device dos not contain an MSSP peripheral for I2C"
$endif      ' _mssp = 1 Or _mssp = 2
$endif      ' _HBUS1_INC_

