$ifndef _HBUS2_INC_
$define _HBUS2_INC_
'
' Routines to communicate via I2C using the MSSP2 peripheral
' Written by Les Johnson for the Proton BASIC Compiler
'
' Setup the default pins used for the MSSP2 peripheral
'
$ifndef Hbus2_SCL_Pin
    $define Hbus2_SCL_Pin PORTB.1
$endif
$ifndef Hbus2_SDA_Pin
    $define Hbus2_SDA_Pin PORTB.2
$endif
'
' Create some variables for the Hbus routines
'
    Dim HbusX_wTimeout As Word System           ' Used for timeout counting
    Dim HbusX_bTemp As Byte System              ' A temporary byte variable
    Dim HbusX_bFlags As Byte System             ' Used for Error flags

$define Hbus2_Error HbusX_bFlags.0              ' Set to 1 if an error occurs
$define Hbus2_tReadWrite HbusX_bFlags.1         ' Set to 1 for read and 0 for write

$define Hbus2_Write 0                           ' Set slave device to write mode
$define Hbus2_Read 1                            ' Set slave device to read mode

$if (_mssp = 2)                                 ' This set of routines if the device has a second MSSP peripheral
    GoTo _Hbus2_Main_                           ' Jump over the routines to the main start
'-----------------------------------------------------------------------------------------------
$define Hbus2_Delay() DelayUS 1

'-----------------------------------------------------------------------------------------------
' Configure the MSSP2 for master I2C mode
' Input     : None
' Output    : None
' Notes     : None
'
$define Hbus2_Setup() _Hbus2_Setup

_Hbus2_Setup Macro-
    GoSub __Hbus2_Setup
Endm

#ifMacro _Hbus2_Setup
__Hbus2_Setup:
    Input Hbus2_SCL_Pin                         ' Make SCL an input
    Input Hbus2_SDA_Pin                         ' Make SDA an input
    SSP2STAT = 128                              ' Disable the slew rate for normal 100KHz operation
                                                ' Set the input levels to conform to I2C specs
    SSP2ADD = ((Xtal * 1000) / (100 * 4)) - 1   ' Calculate the Baud rate value for SSP2ADD
    SSP2CON1 = 40                               ' I2C master mode selected and Enable the SSP2
    Return
#endIfMacro

'-----------------------------------------------------------------------------------------------
' Wait for the I2C bus to become idle
' Input     : None
' Output    : Bit Hbus2_Error holds 1 if failure, else 0
' Notes     : None
'
$define Hbus2_Wait_For_Idle() _Hbus2_Wait_For_Idle

_Hbus2_Wait_For_Idle Macro-
    GoSub __Hbus2_Wait_For_Idle
Endm

#ifMacro- _Hbus2_Wait_For_Idle
__Hbus2_Wait_For_Idle:
    HbusX_wTimeout = $FFFF                  ' Use HbusX_wTimeout as a counting loop
    Repeat
        Hbus2_Delay()                       ' A small delay
        Clear Hbus2_Error                   ' Default to Acknowledge received
        If Byte SSP2CON2 & %00011111 = 0 Then Return ' Zero so I2C bus is ready, so exit with Hbus2_Error clear
        Dec HbusX_wTimeout
    Until HbusX_wTimeout = 0
    Set Hbus2_Error                         ' Set Hbus2_Error to signal no Acknowledge received
    Return
#endIfMacro

'-----------------------------------------------------------------------------------------------
' Generate an I2C Start
' Input     : None
' Output    : None
' Notes     : None
'
$define Hbus2_Start()    '
    Set SSP2CON2bits_SEN '
    While SSP2CON2bits_SEN = 1: Wend

'-----------------------------------------------------------------------------------------------
' Generate an I2C Stop
' Input     : None
' Output    : None
' Notes     : None
'
$define Hbus2_Stop()      '
    Set SSP2CON2bits_PEN  '
    While SSP2CON2bits_PEN = 1: Wend

'-----------------------------------------------------------------------------------------------
' Generate an I2C ReStart
' Input     : None
' Output    : None
' Notes     : None
'
$define Hbus2_ReStart()   '
    Set SSP2CON2bits_RSEN '
    While SSP2CON2bits_RSEN = 1: Wend

'-----------------------------------------------------------------------------------------------
' Generate an I2C Nack
' Input     : None
' Output    : None
' Notes     : None
'
$define Hbus2_Nack() _Hbus2_Nack

_Hbus2_Nack Macro-
    GoSub __Hbus2_Nack
Endm

#ifMacro- _Hbus2_Nack
__Hbus2_Nack:
    Set SSP2CON2bits_ACKDT               ' Set acknowledge bit for Not Ack
    Set SSP2CON2bits_ACKEN               ' Initiate bus Not acknowledge sequence
    While SSP2CON2bits_ACKEN = 1: Wend   ' Check if we are in idle mode
    Return
#endIfMacro

'-----------------------------------------------------------------------------------------------
' Generate an I2C Ack
' Input     : None
' Output    : None
' Notes     : None
'
$define Hbus2_Ack() _Hbus2_Ack

_Hbus2_Ack Macro-
    GoSub __Hbus2_Ack
Endm

#ifMacro- _Hbus2_Ack
__Hbus2_Ack:
    Clear SSP2CON2bits_ACKDT                ' Clear acknowledge bit for Ack
    Set SSP2CON2bits_ACKEN  				' Send an Ack condition
    While SSP2CON2bits_ACKEN = 1: Wend
    Return
#endIfMacro

'-----------------------------------------------------------------------------------------------
' Send a byte via I2C
' Input     : pByteout (SSP2BUF) holds the byte to write
' Output    : Bit Hbus2_Error holds 1 If failure, Else 0
' Notes     : None
'
$define Hbus2_Write_Byte(pByteout)   '
    SSP2BUF = pByteout               '
    While SSP2STATbits_R_W = 1: Wend '
    Hbus2_Wait_For_Idle()

'-----------------------------------------------------------------------------------------------
' Send a slave byte via I2C
' Input     : pSlaveAddress holds the I2C slave address
'           : pReadWrite holds 1 or 0 for read or write (1 = read, 0 = write)
' Output    : Bit Hbus2_Error holds 1 If failure, Else 0
' Notes     : Shifts the address 1 bit left, then sets or clears the read/write bit
'
$define Hbus2_Set_SlaveAddress(pSlaveAddress, pReadWrite) '
    HbusX_bTemp = pSlaveAddress                       '
    Hbus2_tReadWrite = pReadWrite                     '
    _Hbus2_Set_SlaveAddress

_Hbus2_Set_SlaveAddress Macro-
    GoSub __Hbus2_Set_SlaveAddress
Endm

#ifMacro- _Hbus2_Set_SlaveAddress
__Hbus2_Set_SlaveAddress:
    HbusX_bTemp = HbusX_bTemp << 1
    If Hbus2_tReadWrite = Hbus2_Read Then
        Set HbusX_bTemp.0
    EndIf
    SSP2BUF = HbusX_bTemp
    While SSP2STATbits_R_W = 1: Wend            ' Check if data transmit is complete
    Hbus2_Wait_For_Idle()
    Return
#endIfMacro

'-----------------------------------------------------------------------------------------------
' Receive a byte via I2C
' Input     : None
' Output    : pResult (SSP2BUF) holds the byte received
' Notes     : None
'
$define Hbus2_Read_Byte(pResult) '
    Set SSP2CON2bits_RCEN        '
    Hbus2_Wait_For_Idle()        '
    pResult = SSP2BUF

'-----------------------------------------------------------------------------------------------
' Setup the I2C peripherals
'
_Hbus2_Main_:
    Hbus2_Setup()
$else
    $error "Device dos not contain a second MSSP peripheral for I2C"
$endif      ' _mssp = 2
$endif      ' _HBUS2_INC_
